package com.runstate.pithy;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class PithyCommand {

	PithyDB pithydb;

	/** Creates a new instance of PithyCommand */
	public PithyCommand(PithyDB pithydb) {
		this.pithydb = pithydb;
	}

	Pattern addcatexp = Pattern.compile("add ([a-zA-Z]+) \"([^\"]*)\"");

	Pattern addexp = Pattern.compile("add \"([^\"]*)\"");

	Pattern getexp = Pattern.compile("get ([a-zA-Z]+)|get");

	Pattern loadexp = Pattern.compile("load ([^ ]*)$");

	public void run() {
		try {
			BufferedReader inbr = new BufferedReader(new InputStreamReader(
					System.in));

			boolean quit = false;

			while (!quit) {
				boolean consumed = false;

				System.out.print("Pithy>");
				String s = inbr.readLine();

				if (s == null) {
					quit = true;
				} else {

					if (!consumed) {
						if (s.toLowerCase().equals("start")) {
							pithydb.start();
							consumed = true;
						} else if (s.toLowerCase().equals("stop")) {
							pithydb.stop();
							consumed = true;
						} else if (s.toLowerCase().equals("quit")) {
							quit = true;
							consumed = true;
						}
					}

					if (!consumed) {
						consumed = processCommand(s);
					}

					if (!consumed) {
						System.out.println("Command not recognised");
					}

				}
			}

		} catch (IOException ioe) {
			System.out.println("IO Exception caught");
		} finally {
			System.out.println("Closing down");
		}
	}

	private boolean processCommand(String s) {
		boolean consumed = false;

		if (!pithydb.isAvailable()) {
			System.out.println("The database is not available");
			return consumed;
		}

		if (!consumed) {
			Matcher m = addexp.matcher(s);

			if (m.matches()) {
				if (m.groupCount() == 1) {
					Pith p = new Pith("", m.group(1));
					Long id = pithydb.add(p);
					System.out.println("Added with id " + id);
					consumed = true;
				}
			}
		}

		if (!consumed) {
			Matcher m = addcatexp.matcher(s);

			if (m.matches()) {
				Pith p = new Pith(m.group(1), m.group(2));
				Long id = pithydb.add(p);
				System.out.println("Added with id " + id);
				consumed = true;
			}
		}

		if (!consumed) {
			Matcher m = getexp.matcher(s);
			if (m.matches()) {
				ArrayList al;

				if (m.groupCount() == 1) {
					al = pithydb.get(m.group(1));
					if (al == null) {
						System.out.println("No category " + m.group(1));
					} else {
						Pith p = getRandomPith(al);
						System.out.println(p);
					}
				} else {
					al = pithydb.get(null);

					Pith p = getRandomPith(al);
					System.out.println(p);
				}

				consumed = true;
			}
		}

		if (!consumed) {
			Matcher m = loadexp.matcher(s);
			if (m.matches()) {
				File f = new File(m.group(1));
				try {
					BufferedReader br = new BufferedReader(new FileReader(f));

					String ls;

					while ((ls = br.readLine()) != null)
						processCommand(ls);

					System.out.println("Loaded");
				} catch (FileNotFoundException fnfe) {
					System.out.println("File not found");
				} catch (IOException ioe) {
					System.out.println("Problem reading file");
				}
				consumed = true;
			}
		}

		return consumed;
	}

	Random random = new Random();

	private Pith getRandomPith(ArrayList al) {
		if (al.size() == 0)
			return null;
		int i = random.nextInt(al.size());
		return (Pith) al.get(i);
	}
}
