package com.runstate.pithy;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

public class PithyDBDerby implements PithyDB {

	Connection conn;

	Statement stmt;

	PreparedStatement add_stmt;

	PreparedStatement get_stmt;

	boolean available = false;

	public String getURL() {
		return "jdbc:derby:pithderby/pithydb";
	}

	public String getUser() {
		return "sa";
	}

	public String getPassword() {
		return "";
	}

	String pithtable = "pithtable";

	public PithyDBDerby() {
		System.out.println("PithyDBDerby in use");
	}

	public void start() {
		if (available) {
			System.out.println("Already started");
			return;
		}

		try {
			Class.forName("org.apache.derby.jdbc.EmbeddedDriver").newInstance();

			conn = DriverManager.getConnection(getURL() + ";create=true",
					getUser(), getPassword());

			stmt = conn.createStatement();

			DatabaseMetaData dbmd = conn.getMetaData();
			ResultSet rs = dbmd.getTables(null, null, "%", null);

			boolean found = false;

			while (rs.next()) {
				String s = rs.getString(3);
				if (s.equalsIgnoreCase(pithtable)) {
					found = true;
				}
			}

			if (!found) {

				stmt
						.executeUpdate("CREATE TABLE "
								+ pithtable
								+ " ("
								+ "id INTEGER GENERATED ALWAYS AS IDENTITY(START WITH 1000) PRIMARY KEY,"
								+ "category VARCHAR(40),"
								+ "pith VARCHAR(1024) )");
			}

			add_stmt = conn.prepareStatement("insert into " + pithtable
					+ " (category,pith) values (?,?)");
			get_stmt = conn.prepareStatement("select * from " + pithtable
					+ " where category like ?");

			available = true;

		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
	}

	public void stop() {
		if (!available) {
			System.out.println("Already stopped");
			return;
		}

		try {
			conn.close();
			try {
				DriverManager.getConnection(getURL() + ";shutdown=true");
			} catch (SQLException e) {
//				 Silently drop the exception here
				conn=null;
				stmt=null;
//				Attempt to clear out the EmbeddedDriver
				System.gc();
				
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		available = false;
	}

	public boolean isAvailable() {
		return available;
	}

	public Long add(Pith p) {
		if (!available)
			return null;

		try {
			add_stmt.clearParameters();
			add_stmt.setString(1, p.getCategory());
			add_stmt.setString(2, p.getPithString());
			add_stmt.executeUpdate();

			ResultSet rs = stmt.executeQuery("values IDENTITY_VAL_LOCAL()");

			if (rs.next()) {
				return new Long(rs.getLong(1));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return null;

	}

	public ArrayList get(String catagory) {
		if (!available)
			return null;

		try {
			get_stmt.clearParameters();
			if (catagory == null) {
				get_stmt.setString(1, "%");
			} else {
				get_stmt.setString(1, catagory);
			}

			ResultSet rs = get_stmt.executeQuery();

			ArrayList pithresults = new ArrayList();

			while (rs.next()) {
				Pith p = new Pith(new Long(rs.getLong(1)), rs.getString(2), rs
						.getString(3));
				pithresults.add(p);
			}

			return pithresults;
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}
	}

}
