package com.runstate.pithy;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

public class PithyDBHSQLDB implements PithyDB {

	Connection conn;
	Statement stmt;
	PreparedStatement add_stmt;
	PreparedStatement get_stmt;

	boolean available=false;
	
	public String getURL() {
		return "jdbc:hsqldb:pithhsqldb/pithydb";
		//return "jdbc:hsqldb:hsql://localhost";
	}

	public String getUser() {
		return "sa";
	}

	public String getPassword() {
		return "";
	}

	String pithtable = "pithtable";

	public PithyDBHSQLDB() {
		System.out.println("PithyDBHSQLDB in use");
	}
	
	public void start()
	{
		if(available)
		{
			System.out.println("Already started");
			return;
		}
		
		try {
			Class.forName("org.hsqldb.jdbcDriver").newInstance();

			conn = DriverManager.getConnection(getURL(), getUser(),
					getPassword());
			
			stmt = conn.createStatement();
			
			DatabaseMetaData dbmd = conn.getMetaData();
			ResultSet rs = dbmd.getTables(null, null, "%", null);

			boolean found = false;

			while (rs.next()) {
				String s = rs.getString(3);
				if (s.equalsIgnoreCase(pithtable)) {
					found = true;
				}
			}

			if (!found) {

				stmt.executeUpdate("CREATE CACHED TABLE "
								+ pithtable
								+ " ("
								+ "id INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1000) PRIMARY KEY,"
								+ "category VARCHAR(40),"
								+ "pith VARCHAR(1024) );");
			}
			
			stmt.execute("SET WRITE_DELAY FALSE;");
			
			add_stmt = conn.prepareStatement("insert into " + pithtable
					+ " (id,category,pith) values (NULL,?,?);");
			get_stmt = conn.prepareStatement("select * from "+pithtable+" where category like ?;");
			
			available=true;

		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
	}

	public void stop()
	{
		if(!available)
		{
			System.out.println("Already stopped");
			return;
		}
		
		try {
			stmt.execute("shutdown;");
			conn.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		available=false;
	}
	
	public boolean isAvailable()
	{
		return available;
	}
	

	public Long add(Pith p) {
		if(!available) return null;
		
		try {
			add_stmt.clearParameters();
			add_stmt.setString(1,p.getCategory());
			add_stmt.setString(2,p.getPithString());
			add_stmt.executeUpdate();
			
			ResultSet rs=stmt.executeQuery("CALL IDENTITY();");
			
			if(rs.next())
			{
				return new Long(rs.getLong(1));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return null;

	}

	
	public ArrayList get(String catagory) {
		if(!available) return null;
		
		try
		{
			get_stmt.clearParameters();
			if(catagory==null)
			{
				get_stmt.setString(1,"%");
			}
			else
			{
				get_stmt.setString(1,catagory);
			}
			
			ResultSet rs=get_stmt.executeQuery();
			
			ArrayList pithresults=new ArrayList();
			
			while(rs.next())
			{
				Pith p=new Pith(new Long(rs.getLong(1)),rs.getString(2),rs.getString(3));
				pithresults.add(p);
			}
			
			return pithresults;
		}
		catch(SQLException e)
		{
			e.printStackTrace();
			return null;
		}
	}

}
